@@ -1,5 +1,5 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include "milter-enum-types.h"
 #include "milter-protocol.h"
@@ -384,6 +384,34 @@ milter_utils_pipe_mode_get_type (void)
   return etype;
 }
 GType
+milter_enum_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_ENUM_ERROR_NULL_NAME, "MILTER_ENUM_ERROR_NULL_NAME", "null-name" },
+      { MILTER_ENUM_ERROR_UNKNOWN_NAME, "MILTER_ENUM_ERROR_UNKNOWN_NAME", "unknown-name" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterEnumError", values);
+  }
+  return etype;
+}
+GType
+milter_flags_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_FLAGS_ERROR_NULL_NAME, "MILTER_FLAGS_ERROR_NULL_NAME", "null-name" },
+      { MILTER_FLAGS_ERROR_UNKNOWN_NAMES, "MILTER_FLAGS_ERROR_UNKNOWN_NAMES", "unknown-names" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterFlagsError", values);
+  }
+  return etype;
+}
+GType
 milter_connection_error_get_type (void)
 {
   static GType etype = 0;
@@ -470,5 +498,5 @@ milter_event_loop_error_get_type (void)
   return etype;
 }
 
-/* Generated data ends here */
+
 
