@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -33,14 +33,34 @@
 #include "milter-enum-types.h"
 
 #define DEFAULT_KEY "default"
-#define DEFAULT_LEVEL                                           \
-    (MILTER_LOG_LEVEL_CRITICAL | MILTER_LOG_LEVEL_ERROR)
+#define DEFAULT_LEVEL                           \
+    (MILTER_LOG_LEVEL_CRITICAL |                \
+     MILTER_LOG_LEVEL_ERROR |                   \
+     MILTER_LOG_LEVEL_WARNING |                 \
+     MILTER_LOG_LEVEL_MESSAGE |                 \
+     MILTER_LOG_LEVEL_STATISTICS)
 
 #define MILTER_LOGGER_GET_PRIVATE(obj)                  \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj),                 \
                                  MILTER_TYPE_LOGGER,    \
                                  MilterLoggerPrivate))
 
+#define INTERNAL_INTERESTING_LOG_LEVEL                                  \
+    (milter_logger_get_interesting_level(singleton_milter_logger))
+
+#define INTERNAL_LOG(level, format, ...) do {           \
+    if (INTERNAL_INTERESTING_LOG_LEVEL & (level)) {     \
+        milter_logger_log(singleton_milter_logger,      \
+                          MILTER_LOG_DOMAIN,            \
+                          (level),                      \
+                          __FILE__,                     \
+                          __LINE__,                     \
+                          G_STRFUNC,                    \
+                          format, ## __VA_ARGS__);      \
+    }                                                   \
+} while (0)
+
+
 typedef struct _MilterLoggerPrivate	MilterLoggerPrivate;
 struct _MilterLoggerPrivate
 {
@@ -208,21 +228,23 @@ get_property (GObject    *object,
 }
 
 MilterLogLevelFlags
-milter_log_level_flags_from_string (const gchar *level_name)
+milter_log_level_flags_from_string (const gchar *level_name, GError **error)
 {
     if (g_str_equal(level_name, "all")) {
         return MILTER_LOG_LEVEL_ALL;
     } else {
         return milter_utils_flags_from_string(MILTER_TYPE_LOG_LEVEL_FLAGS,
-                                              level_name);
+                                              level_name,
+                                              error);
     }
 }
 
 MilterLogItemFlags
-milter_log_item_flags_from_string (const gchar *item_name)
+milter_log_item_flags_from_string (const gchar *item_name, GError **error)
 {
     return milter_utils_flags_from_string(MILTER_TYPE_LOG_ITEM_FLAGS,
-                                          item_name);
+                                          item_name,
+                                          error);
 }
 
 #define BLACK_COLOR "\033[01;30m"
@@ -295,7 +317,8 @@ log_message (GString *log, MilterLogLevelFlags level, const gchar *message)
     colorize_type = g_getenv("MILTER_LOG_COLORIZE");
     if (colorize_type)
         colorize = milter_utils_enum_from_string(MILTER_TYPE_LOG_COLORIZE,
-                                                 colorize_type);
+                                                 colorize_type,
+                                                 NULL);
 
     if (colorize == MILTER_LOG_COLORIZE_DEFAULT) {
 
@@ -317,6 +340,22 @@ log_message (GString *log, MilterLogLevelFlags level, const gchar *message)
     }
 }
 
+static inline void
+check_milter_debug (MilterLogLevelFlags level)
+{
+    if (MILTER_LOG_LEVEL_CRITICAL <= level &&
+        level <= MILTER_LOG_LEVEL_WARNING) {
+        const gchar *milter_debug;
+
+        milter_debug = g_getenv("MILTER_DEBUG");
+        if (milter_debug &&
+            (strcmp(milter_debug, "fatal-warnings") == 0 ||
+             strcmp(milter_debug, "fatal_warnings") == 0)) {
+            abort();
+        }
+    }
+}
+
 void
 milter_logger_default_log_handler (MilterLogger *logger, const gchar *domain,
                                    MilterLogLevelFlags level,
@@ -335,6 +374,8 @@ milter_logger_default_log_handler (MilterLogger *logger, const gchar *domain,
     if (!(level & target_level))
         return;
 
+    check_milter_debug(level);
+
     log = g_string_new(NULL);
 
     target_item = priv->target_item;
@@ -402,12 +443,26 @@ MilterLogger *
 milter_logger (void)
 {
     if (!singleton_milter_logger) {
+        GError *error = NULL;
+
         singleton_milter_logger = milter_logger_new();
-        milter_logger_set_target_level_by_string(singleton_milter_logger,
-                                                 g_getenv("MILTER_LOG_LEVEL"));
-        milter_logger_set_target_item_by_string(singleton_milter_logger,
-                                                g_getenv("MILTER_LOG_ITEM"));
         milter_logger_connect_default_handler(singleton_milter_logger);
+        if (!milter_logger_set_target_level_by_string(singleton_milter_logger,
+                                                      g_getenv("MILTER_LOG_LEVEL"),
+                                                      &error)) {
+            INTERNAL_LOG(MILTER_LOG_LEVEL_WARNING,
+                         "[logger][level][set][warning] %s", error->message);
+            g_error_free(error);
+            error = NULL;
+        }
+        if (!milter_logger_set_target_item_by_string(singleton_milter_logger,
+                                                     g_getenv("MILTER_LOG_ITEM"),
+                                                     &error)) {
+            INTERNAL_LOG(MILTER_LOG_LEVEL_WARNING,
+                         "[logger][item][set][warning] %s", error->message);
+            g_error_free(error);
+            error = NULL;
+        }
     }
 
     return singleton_milter_logger;
@@ -480,18 +535,25 @@ milter_logger_set_target_level (MilterLogger *logger,
     milter_logger_set_interesting_level(logger, DEFAULT_KEY, interesting_level);
 }
 
-void
+gboolean
 milter_logger_set_target_level_by_string (MilterLogger *logger,
-                                          const gchar *level_name)
+                                          const gchar *level_name,
+                                          GError **error)
 {
     MilterLogLevelFlags level;
 
     if (level_name) {
-        level = milter_log_level_flags_from_string(level_name);
+        GError *local_error = NULL;
+        level = milter_log_level_flags_from_string(level_name, &local_error);
+        if (local_error) {
+            g_propagate_error(error, local_error);
+            return FALSE;
+        }
     } else {
         level = MILTER_LOG_LEVEL_DEFAULT;
     }
     milter_logger_set_target_level(logger, level);
+    return TRUE;
 }
 
 static void
@@ -540,19 +602,25 @@ milter_logger_set_target_item (MilterLogger *logger,
     MILTER_LOGGER_GET_PRIVATE(logger)->target_item = item;
 }
 
-void
+gboolean
 milter_logger_set_target_item_by_string (MilterLogger *logger,
-                                         const gchar *item_name)
+                                         const gchar *item_name,
+                                         GError **error)
 {
     MilterLogItemFlags item;
 
     if (item_name) {
-        item = milter_log_item_flags_from_string(item_name);
+        GError *local_error = NULL;
+        item = milter_log_item_flags_from_string(item_name, &local_error);
+        if (local_error) {
+            g_propagate_error(error, local_error);
+            return FALSE;
+        }
     } else {
         item = MILTER_LOG_ITEM_DEFAULT;
     }
     milter_logger_set_target_item(logger, item);
-
+    return TRUE;
 }
 
 void
