@@ -26,7 +26,6 @@
 #include "milter-syslog-logger.h"
 
 #define INTERESTING_LEVEL_KEY "syslog"
-#define DEFAULT_LEVEL MILTER_LOG_LEVEL_STATISTICS
 
 #define MILTER_SYSLOG_LOGGER_GET_PRIVATE(obj)                   \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj),                         \
@@ -81,8 +80,8 @@ milter_syslog_logger_class_init (MilterSyslogLoggerClass *klass)
     gobject_class->get_property = get_property;
 
     spec = g_param_spec_string("identity",
-                               NULL,
-                               NULL,
+                               "Identity",
+                               "The identity of syslog",
                                NULL,
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
     g_object_class_install_property(gobject_class, PROP_IDENTITY, spec);
@@ -122,7 +121,7 @@ milter_log_level_to_syslog_level (MilterLogLevelFlags milter_log_level)
     else if (milter_log_level & MILTER_LOG_LEVEL_DEBUG)
         return LOG_DEBUG;
     else if (milter_log_level & MILTER_LOG_LEVEL_STATISTICS)
-        return LOG_INFO;
+        return LOG_NOTICE;
     else if (milter_log_level & MILTER_LOG_LEVEL_PROFILE)
         return LOG_DEBUG;
     return LOG_DEBUG;
@@ -141,7 +140,7 @@ cb_log (MilterLogger *logger, const gchar *domain,
 
     target_level = priv->target_level;
     if (target_level == MILTER_LOG_LEVEL_DEFAULT)
-        target_level = DEFAULT_LEVEL;
+        target_level = milter_logger_get_interesting_level(logger);
     if (!(level & target_level))
         return;
 
@@ -256,8 +255,14 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
     priv->logger = milter_logger();
     log_level_env = g_getenv("MILTER_LOG_SYSLOG_LEVEL");
     if (log_level_env) {
-        milter_syslog_logger_set_target_level_by_string(syslog_logger,
-                                                        log_level_env);
+        GError *error = NULL;
+        if (!milter_syslog_logger_set_target_level_by_string(syslog_logger,
+                                                             log_level_env,
+                                                             &error)) {
+            milter_warning("[syslog-logger][level][set][warning] %s",
+                           error->message);
+            g_error_free(error);
+        }
     } else {
         MilterLogLevelFlags log_level;
         log_level = milter_logger_get_target_level(priv->logger);
@@ -379,29 +384,33 @@ milter_syslog_logger_set_target_level (MilterSyslogLogger *logger,
                                        MilterLogLevelFlags level)
 {
     MilterSyslogLoggerPrivate *priv;
-    MilterLogLevelFlags interesting_level = level;
 
     priv = MILTER_SYSLOG_LOGGER_GET_PRIVATE(logger);
     priv->target_level = level;
-    if (interesting_level == MILTER_LOG_LEVEL_DEFAULT)
-        interesting_level = DEFAULT_LEVEL;
     milter_logger_set_interesting_level(priv->logger,
                                         INTERESTING_LEVEL_KEY,
-                                        interesting_level);
+                                        level);
 }
 
-void
+gboolean
 milter_syslog_logger_set_target_level_by_string (MilterSyslogLogger *logger,
-                                                 const gchar *level_name)
+                                                 const gchar *level_name,
+                                                 GError **error)
 {
     MilterLogLevelFlags level;
 
     if (level_name) {
-        level = milter_log_level_flags_from_string(level_name);
+        GError *local_error = NULL;
+        level = milter_log_level_flags_from_string(level_name, &local_error);
+        if (local_error) {
+            g_propagate_error(error, local_error);
+            return FALSE;
+        }
     } else {
         level = MILTER_LOG_LEVEL_DEFAULT;
     }
     milter_syslog_logger_set_target_level(logger, level);
+    return TRUE;
 }
 
 /*
