@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -64,7 +64,7 @@ milter_utils_inspect_enum (GType enum_type, gint enum_value)
 
     enum_class = g_type_class_ref(enum_type);
     if (!enum_class)
-        return g_strdup_printf("unknown enum type: %s(%" G_GSIZE_FORMAT ")",
+        return g_strdup_printf("unknown enum type: %s(%lu)",
                                g_type_name(enum_type), enum_type);
 
     value = g_enum_get_value(enum_class, enum_value);
@@ -93,7 +93,7 @@ milter_utils_get_enum_name (GType enum_type, gint enum_value)
 
     enum_class = g_type_class_ref(enum_type);
     if (!enum_class)
-        return g_strdup_printf("unknown enum type: %s(%" G_GSIZE_FORMAT ")",
+        return g_strdup_printf("unknown enum type: %s(%lu)",
                                g_type_name(enum_type), enum_type);
 
     value = g_enum_get_value(enum_class, enum_value);
@@ -117,7 +117,7 @@ milter_utils_get_enum_nick_name (GType enum_type, gint enum_value)
 
     enum_class = g_type_class_ref(enum_type);
     if (!enum_class)
-        return g_strdup_printf("unknown enum type: %s(%" G_GSIZE_FORMAT ")",
+        return g_strdup_printf("unknown enum type: %s(%lu)",
                                g_type_name(enum_type), enum_type);
 
     value = g_enum_get_value(enum_class, enum_value);
@@ -141,7 +141,7 @@ milter_utils_inspect_flags (GType flags_type, guint flags)
 
     flags_class = g_type_class_ref(flags_type);
     if (!flags_class)
-        return g_strdup_printf("unknown flags type: %s(%" G_GSIZE_FORMAT ")",
+        return g_strdup_printf("unknown flags type: %s(%lu)",
                                g_type_name(flags_type), flags_type);
 
 
@@ -187,7 +187,7 @@ milter_utils_get_flags_names (GType flags_type, guint flags)
 
     flags_class = g_type_class_ref(flags_type);
     if (!flags_class)
-        return g_strdup_printf("unknown flags type: %s(%" G_GSIZE_FORMAT ")",
+        return g_strdup_printf("unknown flags type: %s(%lu)",
                                g_type_name(flags_type), flags_type);
 
     names = g_string_new(NULL);
@@ -335,7 +335,7 @@ milter_utils_macro_stage_to_command (MilterMacroStage stage)
 
 void
 milter_utils_set_error_with_sub_error (GError **error,
-                                       GQuark domain, 
+                                       GQuark domain,
                                        gint error_code,
                                        GError *sub_error,
                                        const gchar *format,
@@ -488,16 +488,30 @@ milter_utils_inspect_list_pointer (const GList *list)
     return g_string_free(inspected, FALSE);
 }
 
+GQuark
+milter_flags_error_quark (void)
+{
+    return g_quark_from_static_string("milter-flags-error-quark");
+}
+
 guint
 milter_utils_flags_from_string (GType        flags_type,
-                                const gchar *flags_string)
+                                const gchar *flags_string,
+                                GError     **error)
 {
     gchar **names, **name;
     GFlagsClass *flags_class;
     guint flags = 0;
+    GString *unknown_strings = NULL;
 
-    if (!flags_string)
+    if (!flags_string) {
+        g_set_error(error,
+                    MILTER_FLAGS_ERROR,
+                    MILTER_FLAGS_ERROR_NULL_NAME,
+                    "flags name is NULL (<%s>)",
+                    g_type_name(flags_type));
         return 0;
+    }
 
     names = g_strsplit(flags_string, "|", 0);
     flags_class = g_type_class_ref(flags_type);
@@ -509,31 +523,70 @@ milter_utils_flags_from_string (GType        flags_type,
             GFlagsValue *value;
 
             value = g_flags_get_value_by_nick(flags_class, *name);
-            if (value)
+            if (value) {
                 flags |= value->value;
+            } else {
+                if (!unknown_strings)
+                    unknown_strings = g_string_new(NULL);
+                g_string_append_printf(unknown_strings,
+                                       "%s<%s>",
+                                       unknown_strings->len > 0 ? "|" : "",
+                                       *name);
+            }
         }
     }
     g_type_class_unref(flags_class);
     g_strfreev(names);
 
+    if (unknown_strings) {
+        g_set_error(error,
+                    MILTER_FLAGS_ERROR,
+                    MILTER_FLAGS_ERROR_UNKNOWN_NAMES,
+                    "unknown flag names: [%s](<%s>)",
+                    unknown_strings->str,
+                    g_type_name(flags_type));
+        g_string_free(unknown_strings, TRUE);
+    }
+
     return flags;
 }
 
+GQuark
+milter_enum_error_quark (void)
+{
+    return g_quark_from_static_string("milter-enum-error-quark");
+}
+
 gint
 milter_utils_enum_from_string (GType        enum_type,
-                               const gchar *enum_string)
+                               const gchar *enum_string,
+                               GError     **error)
 {
     GEnumClass *enum_class;
     GEnumValue *enum_value;
     gint value = 0;
 
-    if (!enum_string)
+    if (!enum_string) {
+        g_set_error(error,
+                    MILTER_ENUM_ERROR,
+                    MILTER_ENUM_ERROR_NULL_NAME,
+                    "enum name is NULL (<%s>)",
+                    g_type_name(enum_type));
         return 0;
+    }
 
     enum_class = g_type_class_ref(enum_type);
     enum_value = g_enum_get_value_by_nick(enum_class, enum_string);
-    if (enum_value)
+    if (enum_value) {
         value = enum_value->value;
+    } else {
+        g_set_error(error,
+                    MILTER_ENUM_ERROR,
+                    MILTER_ENUM_ERROR_UNKNOWN_NAME,
+                    "unknown enum name: <%s>(<%s>)",
+                    enum_string,
+                    g_type_name(enum_type));
+    }
     g_type_class_unref(enum_class);
 
     return value;
@@ -821,6 +874,32 @@ milter_utils_parse_file_mode (const gchar *string,
     return success;
 }
 
+#if GLIB_CHECK_VERSION(2, 14, 0)
+GList *
+milter_utils_hash_table_get_keys (GHashTable *table)
+{
+    return g_hash_table_get_keys(table);
+}
+#else
+static void
+collect_key (gpointer key, gpointer value, gpointer user_data)
+{
+    GList **keys = (GList **)user_data;
+
+    *keys = g_list_prepend(*keys, key);
+}
+
+GList *
+milter_utils_hash_table_get_keys (GHashTable *table)
+{
+    GList *keys = NULL;
+
+    g_hash_table_foreach(table, collect_key, &keys);
+
+    return keys;
+}
+#endif
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
