@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -113,10 +113,12 @@ void             milter_utils_set_error_with_sub_error
 
 guint            milter_utils_flags_from_string
                                              (GType        flags_type,
-                                              const gchar *flags_string);
+                                              const gchar *flags_string,
+                                              GError     **error);
 gint             milter_utils_enum_from_string
                                              (GType        enum_type,
-                                              const gchar *enum_string);
+                                              const gchar *enum_string,
+                                              GError     **error);
 
 void             milter_utils_append_indent  (GString *string,
                                               guint    size);
@@ -146,6 +148,8 @@ gboolean         milter_utils_guess_console_color_usability
 gboolean         milter_utils_parse_file_mode(const gchar  *string,
                                               guint        *mode,
                                               gchar       **error_message);
+GList           *milter_utils_hash_table_get_keys
+                                             (GHashTable *table);
 
 typedef enum {
     MILTER_UTILS_READ_PIPE,
@@ -153,6 +157,24 @@ typedef enum {
 } MilterUtilsPipeMode;
 
 
+#define MILTER_ENUM_ERROR           (milter_enum_error_quark())
+#define MILTER_FLAGS_ERROR          (milter_flags_error_quark())
+
+typedef enum
+{
+    MILTER_ENUM_ERROR_NULL_NAME,
+    MILTER_ENUM_ERROR_UNKNOWN_NAME
+} MilterEnumError;
+
+typedef enum
+{
+    MILTER_FLAGS_ERROR_NULL_NAME,
+    MILTER_FLAGS_ERROR_UNKNOWN_NAMES
+} MilterFlagsError;
+
+GQuark           milter_enum_error_quark       (void);
+GQuark           milter_flags_error_quark      (void);
+
 
 G_END_DECLS
 
