@@ -436,7 +436,7 @@ milter_writer_write (MilterWriter *writer, const gchar *chunk, gsize chunk_size,
         const gchar *message = "no write channel";
         g_set_error(error,
                     MILTER_WRITER_ERROR, MILTER_WRITER_ERROR_NO_CHANNEL,
-                    message);
+                    "%s", message);
         milter_error("[%u] [writer][write][error] %s",
                      priv->tag, message);
         return FALSE;
@@ -446,7 +446,7 @@ milter_writer_write (MilterWriter *writer, const gchar *chunk, gsize chunk_size,
         const gchar *message = "can't write to not started or shutdown channel";
         g_set_error(error,
                     MILTER_WRITER_ERROR, MILTER_WRITER_ERROR_NOT_READY,
-                    message);
+                    "%s", message);
         milter_error("[%u] [writer][write][error] %s",
                      priv->tag, message);
         return FALSE;
@@ -487,7 +487,7 @@ milter_writer_flush (MilterWriter *writer, GError **error)
         const gchar *message = "no write channel";
         g_set_error(error,
                     MILTER_WRITER_ERROR, MILTER_WRITER_ERROR_NO_CHANNEL,
-                    message);
+                    "%s", message);
         milter_error("[%u] [writer][flush][error] %s",
                      priv->tag, message);
         return FALSE;
@@ -497,7 +497,7 @@ milter_writer_flush (MilterWriter *writer, GError **error)
         const gchar *message = "can't flush to not started or shutdown channel";
         g_set_error(error,
                     MILTER_WRITER_ERROR, MILTER_WRITER_ERROR_NOT_READY,
-                    message);
+                    "%s", message);
         milter_error("[%u] [writer][flush][error] %s",
                      priv->tag, message);
         return FALSE;
@@ -557,13 +557,21 @@ watch_io_channel (MilterWriter *writer, MilterEventLoop *loop)
 
     priv = MILTER_WRITER_GET_PRIVATE(writer);
 
-    priv->loop = loop;
-    g_object_ref(priv->loop);
     priv->error_watch_id =
         milter_event_loop_watch_io(loop,
                                    priv->io_channel,
                                    G_IO_ERR | G_IO_HUP | G_IO_NVAL,
                                    error_watch_func, writer);
+    if (priv->error_watch_id > 0) {
+        g_object_ref(loop);
+        if (priv->loop) {
+            g_object_unref(priv->loop);
+        }
+        priv->loop = loop;
+    } else {
+        milter_error("[%u] [writer][watch][fail] TODO: raise error",
+                     priv->tag);
+    }
 
     milter_debug("[%u] [writer][watch] %u", priv->tag, priv->error_watch_id);
 }
