@@ -78,6 +78,8 @@ struct _MilterManagerConfigurationPrivate
     MilterClientEventLoopBackend event_loop_backend;
     guint n_workers;
     guint default_packet_buffer_size;
+    gboolean use_syslog;
+    gchar *syslog_facility;
 };
 
 enum
@@ -109,8 +111,11 @@ enum
     PROP_CUSTOM_CONFIGURATION_DIRECTORY,
     PROP_CONNECTION_CHECK_INTERVAL,
     PROP_EVENT_LOOP_BACKEND,
-    PROP_N_WORKER_PROCESSES,
-    PROP_DEFAULT_PACKET_BUFFER_SIZE
+    PROP_N_WORKERS,
+    PROP_DEFAULT_PACKET_BUFFER_SIZE,
+    PROP_PREFIX,
+    PROP_USE_SYSLOG,
+    PROP_SYSLOG_FACILITY
 };
 
 enum
@@ -406,7 +411,7 @@ milter_manager_configuration_class_init (MilterManagerConfigurationClass *klass)
                              "The number of worker processes of the client",
                              0, MILTER_CLIENT_MAX_N_WORKERS, 0,
                              G_PARAM_READWRITE);
-    g_object_class_install_property(gobject_class, PROP_N_WORKER_PROCESSES, spec);
+    g_object_class_install_property(gobject_class, PROP_N_WORKERS, spec);
 
     spec = g_param_spec_uint("default-packet-buffer-size",
                              "Default packet buffer size",
@@ -417,6 +422,26 @@ milter_manager_configuration_class_init (MilterManagerConfigurationClass *klass)
     g_object_class_install_property(gobject_class,
                                     PROP_DEFAULT_PACKET_BUFFER_SIZE, spec);
 
+    spec = g_param_spec_string("prefix",
+                               "Prefix",
+                               "The prefix of install directory",
+                               NULL,
+                               G_PARAM_READABLE);
+    g_object_class_install_property(gobject_class, PROP_PREFIX, spec);
+
+    spec = g_param_spec_boolean("use-syslog",
+                                "Use Syslog",
+                                "Whether milter-manager uses syslog",
+                                TRUE,
+                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+    g_object_class_install_property(gobject_class, PROP_USE_SYSLOG, spec);
+
+    spec = g_param_spec_string("syslog-facility",
+                               "Syslog Facility",
+                               "The syslog facility of the milter-manager",
+                               NULL,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+    g_object_class_install_property(gobject_class, PROP_SYSLOG_FACILITY, spec);
 
     signals[CONNECTED] =
         g_signal_new("connected",
@@ -469,15 +494,13 @@ milter_manager_configuration_init (MilterManagerConfiguration *configuration)
     priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
     priv->n_workers = 0;
     priv->default_packet_buffer_size = 0;
+    priv->syslog_facility = NULL;
 
     config_dir_env = g_getenv("MILTER_MANAGER_CONFIG_DIR");
     if (config_dir_env)
         priv->load_paths = g_list_append(priv->load_paths,
                                          g_strdup(config_dir_env));
     priv->load_paths = g_list_append(priv->load_paths, g_strdup(CONFIG_DIR));
-
-
-    milter_manager_configuration_clear(configuration);
 }
 
 static void
@@ -485,12 +508,18 @@ dispose (GObject *object)
 {
     MilterManagerConfiguration *configuration;
     MilterManagerConfigurationPrivate *priv;
+    GError *error = NULL;
 
     configuration = MILTER_MANAGER_CONFIGURATION(object);
     priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
 
     milter_manager_configuration_clear_load_paths(configuration);
-    milter_manager_configuration_clear(configuration);
+    milter_manager_configuration_clear(configuration, &error);
+    if (error) {
+        milter_error("[configuration][dispose][clear][error] %s",
+                     error->message);
+        g_error_free(error);
+    }
 
     if (priv->manager_connection_spec) {
         g_free(priv->manager_connection_spec);
@@ -619,7 +648,7 @@ set_property (GObject      *object,
         milter_manager_configuration_set_event_loop_backend(
             config, g_value_get_enum(value));
         break;
-    case PROP_N_WORKER_PROCESSES:
+    case PROP_N_WORKERS:
         milter_manager_configuration_set_n_workers(config, g_value_get_uint(value));
         break;
     case PROP_DEFAULT_PACKET_BUFFER_SIZE:
@@ -627,6 +656,14 @@ set_property (GObject      *object,
             config,
             g_value_get_uint(value));
         break;
+    case PROP_USE_SYSLOG:
+        milter_manager_configuration_set_use_syslog(config,
+                                                    g_value_get_boolean(value));
+        break;
+    case PROP_SYSLOG_FACILITY:
+        milter_manager_configuration_set_syslog_facility(
+            config, g_value_get_string(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -721,12 +758,21 @@ get_property (GObject    *object,
     case PROP_EVENT_LOOP_BACKEND:
         g_value_set_enum(value, priv->event_loop_backend);
         break;
-    case PROP_N_WORKER_PROCESSES:
+    case PROP_N_WORKERS:
         g_value_set_uint(value, priv->n_workers);
         break;
     case PROP_DEFAULT_PACKET_BUFFER_SIZE:
         g_value_set_uint(value, priv->default_packet_buffer_size);
         break;
+    case PROP_PREFIX:
+        g_value_set_string(value, PREFIX);
+        break;
+    case PROP_USE_SYSLOG:
+        g_value_set_boolean(value, priv->use_syslog);
+        break;
+    case PROP_SYSLOG_FACILITY:
+        g_value_set_string(value, priv->syslog_facility);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -809,23 +855,59 @@ milter_manager_configuration_error_quark (void)
     return g_quark_from_static_string("milter-manager-configuration-error-quark");
 }
 
-MilterManagerConfiguration *
-milter_manager_configuration_new (const gchar *first_property,
-                                  ...)
+static MilterManagerConfiguration *
+milter_manager_configuration_instantiate_va_list (const gchar *first_property,
+                                                  va_list var_args)
 {
     MilterManagerModule *module;
     GObject *configuration;
-    va_list var_args;
 
     module = milter_manager_configuration_load_module("ruby");
     g_return_val_if_fail(module != NULL, NULL);
 
-    va_start(var_args, first_property);
     configuration = milter_manager_module_instantiate(module,
                                                       first_property, var_args);
+    return MILTER_MANAGER_CONFIGURATION(configuration);
+}
+
+MilterManagerConfiguration *
+milter_manager_configuration_instantiate (const gchar *first_property,
+                                          ...)
+{
+    MilterManagerConfiguration *configuration;
+    va_list var_args;
+
+    va_start(var_args, first_property);
+    configuration =
+        milter_manager_configuration_instantiate_va_list(first_property,
+                                                         var_args);
     va_end(var_args);
 
-    return MILTER_MANAGER_CONFIGURATION(configuration);
+    return configuration;
+}
+
+MilterManagerConfiguration *
+milter_manager_configuration_new (const gchar *first_property,
+                                  ...)
+{
+    MilterManagerConfiguration *configuration;
+    va_list var_args;
+    GError *error = NULL;
+
+    va_start(var_args, first_property);
+    configuration =
+        milter_manager_configuration_instantiate_va_list(first_property,
+                                                         var_args);
+    va_end(var_args);
+
+    milter_manager_configuration_clear(configuration, &error);
+    if (error) {
+        milter_error("[configuration][new][clear][error] %s",
+                     error->message);
+        g_error_free(error);
+    }
+
+    return configuration;
 }
 
 void
@@ -892,7 +974,10 @@ find_file (MilterManagerConfiguration *configuration,
         }
     }
 
-    if (!full_path) {
+    if (full_path) {
+        milter_debug("[configuration][path][found] <%s>", full_path);
+    } else {
+        GError *local_error = NULL;
         GString *inspected_load_paths;
 
         inspected_load_paths = g_string_new("[");
@@ -904,12 +989,15 @@ find_file (MilterManagerConfiguration *configuration,
                 g_string_append(inspected_load_paths, ", ");
         }
         g_string_append(inspected_load_paths, "]");
-        g_set_error(error,
+        g_set_error(&local_error,
                     MILTER_MANAGER_CONFIGURATION_ERROR,
                     MILTER_MANAGER_CONFIGURATION_ERROR_NOT_EXIST,
                     "path %s doesn't exist in load path (%s)",
                     file_name, inspected_load_paths->str);
+        milter_debug("[configuration][path][nonexistent] %s",
+                     local_error->message);
         g_string_free(inspected_load_paths, TRUE);
+        g_propagate_error(error, local_error);
     }
 
     return full_path;
@@ -1011,26 +1099,36 @@ milter_manager_configuration_load_custom_if_exist (MilterManagerConfiguration *c
     return FALSE;
 }
 
-void
-milter_manager_configuration_reload (MilterManagerConfiguration *configuration)
+gboolean
+milter_manager_configuration_reload (MilterManagerConfiguration *configuration,
+                                     GError **error)
 {
-    GError *error = NULL;
+    GError *local_error = NULL;
+
+    if (!milter_manager_configuration_clear(configuration, &local_error)) {
+        milter_error("[configuration][load][clear][error] <%s>: %s",
+                     CONFIG_FILE_NAME, local_error->message);
+        g_propagate_error(error, local_error);
+        return FALSE;
+    }
 
-    milter_manager_configuration_clear(configuration);
     if (!milter_manager_configuration_load(configuration, CONFIG_FILE_NAME,
-                                           &error)) {
-        milter_error("[configuration][error][load] <%s>: %s",
-                     CONFIG_FILE_NAME, error->message);
-        g_error_free(error);
-        return;
+                                           &local_error)) {
+        milter_error("[configuration][load][error] <%s>: %s",
+                     CONFIG_FILE_NAME, local_error->message);
+        g_propagate_error(error, local_error);
+        return FALSE;
     }
 
     if (!milter_manager_configuration_load_custom_if_exist(
-            configuration, CUSTOM_CONFIG_FILE_NAME, &error)) {
-        milter_error("[configuration][error][load][custom] <%s>: %s",
-                     CUSTOM_CONFIG_FILE_NAME, error->message);
-        g_error_free(error);
+            configuration, CUSTOM_CONFIG_FILE_NAME, &local_error)) {
+        milter_error("[configuration][load][custom][error] <%s>: %s",
+                     CUSTOM_CONFIG_FILE_NAME, local_error->message);
+        g_propagate_error(error, local_error);
+        return FALSE;
     }
+
+    return TRUE;
 }
 
 gboolean
@@ -1811,6 +1909,7 @@ clear_manager (MilterManagerConfigurationPrivate *priv)
     priv->max_connections = 0;
     priv->max_file_descriptors = 0;
     priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
+    priv->event_loop_backend = MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB;
     priv->n_workers = 0;
     priv->default_packet_buffer_size = 0;
 }
@@ -1885,8 +1984,19 @@ clear_socket (MilterManagerConfigurationPrivate *priv)
     priv->remove_controller_unix_socket_on_create = TRUE;
 }
 
-void
-milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
+static void
+clear_log (MilterManagerConfigurationPrivate *priv)
+{
+    priv->use_syslog = TRUE;
+    if (priv->syslog_facility) {
+        g_free(priv->syslog_facility);
+        priv->syslog_facility = NULL;
+    }
+}
+
+gboolean
+milter_manager_configuration_clear (MilterManagerConfiguration *configuration,
+                                    GError **error)
 {
     MilterManagerConfigurationPrivate *priv;
     MilterManagerConfigurationClass *configuration_class;
@@ -1903,17 +2013,22 @@ milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
     clear_process(priv);
     clear_configuration(priv);
     clear_socket(priv);
+    clear_log(priv);
 
     configuration_class = MILTER_MANAGER_CONFIGURATION_GET_CLASS(configuration);
     if (configuration_class->clear) {
-        GError *error = NULL;
+        GError *local_error = NULL;
 
-        milter_debug("[configuration][clear]");
-        if (!configuration_class->clear(configuration, &error)) {
-            milter_error("[configuration][error][clear] %s", error->message);
-            g_error_free(error);
+        milter_debug("[configuration][clear][custom]");
+        if (!configuration_class->clear(configuration, &local_error)) {
+            milter_error("[configuration][clear][custom][error] %s",
+                         local_error->message);
+            g_propagate_error(error, local_error);
+            return FALSE;
         }
     }
+
+    return TRUE;
 }
 
 void
@@ -1927,7 +2042,7 @@ milter_manager_configuration_maintain (MilterManagerConfiguration *configuration
 
         milter_debug("[configuration][maintain]");
         if (!configuration_class->maintain(configuration, &error)) {
-            milter_error("[configuration][error][maintain] %s", error->message);
+            milter_error("[configuration][maintain][error] %s", error->message);
             g_error_free(error);
         }
     }
@@ -2163,6 +2278,53 @@ milter_manager_configuration_set_default_packet_buffer_size (MilterManagerConfig
     priv->default_packet_buffer_size = size;
 }
 
+const gchar *
+milter_manager_configuration_get_prefix (MilterManagerConfiguration *configuration)
+{
+    return PREFIX;
+}
+
+gboolean
+milter_manager_configuration_get_use_syslog (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->use_syslog;
+}
+
+void
+milter_manager_configuration_set_use_syslog (MilterManagerConfiguration *configuration,
+                                             gboolean                    use_syslog)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->use_syslog = use_syslog;
+}
+
+const gchar *
+milter_manager_configuration_get_syslog_facility (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->syslog_facility;
+}
+
+void
+milter_manager_configuration_set_syslog_facility (MilterManagerConfiguration *configuration,
+                                                  const gchar                *facility)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    if (priv->syslog_facility) {
+        g_free(priv->syslog_facility);
+    }
+    priv->syslog_facility = g_strdup(facility);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
