@@ -81,6 +81,9 @@ void         _milter_manager_configuration_init        (void);
 void         _milter_manager_configuration_quit        (void);
 
 
+MilterManagerConfiguration *
+              milter_manager_configuration_instantiate (const gchar *first_property,
+                                                        ...);
 MilterManagerConfiguration *
               milter_manager_configuration_new         (const gchar *first_property,
                                                         ...);
@@ -111,8 +114,9 @@ gboolean      milter_manager_configuration_load_custom_if_exist
                                      (MilterManagerConfiguration *configuration,
                                       const gchar                *file_name,
                                       GError                    **error);
-void          milter_manager_configuration_reload
-                                     (MilterManagerConfiguration *configuration);
+gboolean      milter_manager_configuration_reload
+                                     (MilterManagerConfiguration *configuration,
+                                      GError                    **error);
 
 gboolean      milter_manager_configuration_save_custom
                                      (MilterManagerConfiguration *configuration,
@@ -295,8 +299,9 @@ void          milter_manager_configuration_remove_applicable_condition_by_name
 void          milter_manager_configuration_clear_applicable_conditions
                                      (MilterManagerConfiguration *configuration);
 
-void          milter_manager_configuration_clear
-                                     (MilterManagerConfiguration *configuration);
+gboolean      milter_manager_configuration_clear
+                                     (MilterManagerConfiguration *configuration,
+                                      GError                    **error);
 
 void          milter_manager_configuration_session_finished
                                      (MilterManagerConfiguration *configuration);
@@ -352,6 +357,21 @@ void          milter_manager_configuration_set_default_packet_buffer_size
                                      (MilterManagerConfiguration *configuration,
                                       guint                       size);
 
+const gchar  *milter_manager_configuration_get_prefix
+                                     (MilterManagerConfiguration *configuration);
+
+gboolean      milter_manager_configuration_get_use_syslog
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_use_syslog
+                                     (MilterManagerConfiguration *configuration,
+                                      gboolean                    use_syslog);
+
+const gchar  *milter_manager_configuration_get_syslog_facility
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_syslog_facility
+                                     (MilterManagerConfiguration *configuration,
+                                      const gchar                *facility);
+
 G_END_DECLS
 
 #endif /* __MILTER_MANAGER_CONFIGURATION_H__ */
