@@ -194,9 +194,7 @@ set_property (GObject      *object,
     case PROP_EVENT_LOOP:
         if (priv->event_loop)
             g_object_unref(priv->event_loop);
-        priv->event_loop = g_value_get_object(value);
-        if (priv->event_loop)
-            g_object_ref(priv->event_loop);
+        priv->event_loop = g_value_dup_object(value);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -459,12 +457,8 @@ milter_manager_controller_listen (MilterManagerController *controller,
     config = milter_manager_get_configuration(priv->manager);
     spec = milter_manager_configuration_get_controller_connection_spec(config);
     if (!spec) {
-        local_error = g_error_new(MILTER_MANAGER_CONTROLLER_ERROR,
-                                  MILTER_MANAGER_CONTROLLER_ERROR_NO_SPEC,
-                                  "connection spec isn't specified");
-        milter_error("[controller][error][listen] %s", local_error->message);
-        g_propagate_error(error, local_error);
-        return FALSE;
+        milter_info("[controller][disabled] connection spec isn't specified");
+        return TRUE;
     }
 
     remove_socket = milter_manager_configuration_is_remove_controller_unix_socket_on_create(config);
