@@ -43,15 +43,8 @@
 
 static gboolean initialized = FALSE;
 static MilterManager *the_manager = NULL;
-static gchar *option_spec = NULL;
 static gchar *option_config_dir = NULL;
-static gchar *option_pid_file = NULL;
-static gchar *option_user_name = NULL;
-static gchar *option_group_name = NULL;
-static gchar *option_socket_group_name = NULL;
-static gboolean option_daemon = FALSE;
 static gboolean option_show_config = FALSE;
-static gboolean option_verbose = FALSE;
 
 static gboolean io_detached = FALSE;
 
@@ -88,62 +81,14 @@ print_version (const gchar *option_name, const gchar *value,
     return TRUE;
 }
 
-static gboolean
-parse_spec_arg (const gchar *option_name,
-                const gchar *value,
-                gpointer data,
-                GError **error)
-{
-    GError *spec_error = NULL;
-    gboolean success;
-
-    success = milter_connection_parse_spec(value, NULL, NULL, NULL, &spec_error);
-    if (success) {
-        if (option_spec)
-            g_free(option_spec);
-        option_spec = g_strdup(value);
-    } else {
-        g_set_error(error,
-                    G_OPTION_ERROR,
-                    G_OPTION_ERROR_BAD_VALUE,
-                    "invalid connection spec: %s",
-                    spec_error->message);
-        g_error_free(spec_error);
-    }
-
-    return success;
-}
-
 static const GOptionEntry option_entries[] =
 {
-    {"connection-spec", 's', 0, G_OPTION_ARG_CALLBACK, parse_spec_arg,
-     N_("The spec of socket. (unix:PATH|inet:PORT[@HOST]|inet6:PORT[@HOST])"),
-     "SPEC"},
     {"config-dir", 'c',
-     G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING, &option_config_dir,
+     0, G_OPTION_ARG_FILENAME, &option_config_dir,
      N_("The configuration directory that has configuration file."),
      "DIRECTORY"},
-    {"pid-file", 0,
-     G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING, &option_pid_file,
-     N_("The file name to be saved PID."),
-     "FILE"},
-    {"user-name", 'u', 0, G_OPTION_ARG_STRING, &option_user_name,
-     N_("The user name for running milter-manager."),
-     "NAME"},
-    {"group-name", 'g', 0, G_OPTION_ARG_STRING, &option_group_name,
-     N_("The group name for running milter-manager."),
-     "NAME"},
-    {"socket-group-name", 0, 0, G_OPTION_ARG_STRING, &option_socket_group_name,
-     N_("The group name for UNIX domain socket."),
-     "NAME"},
-    {"daemon", 0, 0, G_OPTION_ARG_NONE, &option_daemon,
-     N_("Run as daemon process."), NULL},
-    {"no-daemon", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &option_daemon,
-     N_("Cancel the prior --daemon options."), NULL},
     {"show-config", 0, 0, G_OPTION_ARG_NONE, &option_show_config,
      N_("Show configuration and exit"), NULL},
-    {"verbose", 0, 0, G_OPTION_ARG_NONE, &option_verbose,
-     N_("Be verbose"), NULL},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
      N_("Show version"), NULL},
     {NULL}
@@ -153,8 +98,10 @@ void
 milter_manager_init (int *argc, char ***argv)
 {
     GOptionContext *option_context;
-    GOptionGroup *main_group;
+    GOptionGroup *milter_group;
     GError *error = NULL;
+    MilterManager *manager;
+    MilterManagerConfiguration *config;
 
     if (initialized)
         return;
@@ -171,16 +118,16 @@ milter_manager_init (int *argc, char ***argv)
     milter_server_init();
     milter_manager_log_handler_id = MILTER_GLIB_LOG_DELEGATE("milter-manager");
 
-    if (milter_get_log_level() == MILTER_LOG_LEVEL_DEFAULT) {
-        milter_set_log_level(MILTER_LOG_LEVEL_CRITICAL |
-                             MILTER_LOG_LEVEL_ERROR |
-                             MILTER_LOG_LEVEL_WARNING |
-                             MILTER_LOG_LEVEL_STATISTICS);
-    }
-
     option_context = g_option_context_new(NULL);
-    g_option_context_add_main_entries(option_context, option_entries, NULL);
-    main_group = g_option_context_get_main_group(option_context);
+    g_option_context_add_main_entries(option_context, option_entries,
+                                      GETTEXT_PACKAGE);
+
+    _milter_manager_configuration_init();
+    config = milter_manager_configuration_new(NULL);
+    manager = milter_manager_new(config);
+    g_object_unref(config);
+    milter_group = milter_client_get_option_group(MILTER_CLIENT(manager));
+    g_option_context_add_group(option_context, milter_group);
 
     if (!g_option_context_parse(option_context, argc, argv, &error)) {
         g_print("%s\n", error->message);
@@ -196,17 +143,14 @@ milter_manager_init (int *argc, char ***argv)
 #endif
 
         g_error_free(error);
-
         g_option_context_free(option_context);
+        g_object_unref(manager);
 
         exit(EXIT_FAILURE);
     }
 
-    if (option_verbose)
-        milter_set_log_level(MILTER_LOG_LEVEL_ALL);
-
-    _milter_manager_configuration_init();
     g_option_context_free(option_context);
+    the_manager = manager;
 }
 
 void
@@ -215,13 +159,13 @@ milter_manager_quit (void)
     if (!initialized)
         return;
 
-    _milter_manager_configuration_quit();
-
-    if (option_spec) {
-        g_free(option_spec);
-        option_spec = NULL;
+    if (the_manager) {
+        g_object_unref(the_manager);
+        the_manager = NULL;
     }
 
+    _milter_manager_configuration_quit();
+
     g_log_remove_handler("milter-manager", milter_manager_log_handler_id);
     milter_server_quit();
     milter_client_quit();
@@ -258,11 +202,13 @@ static gboolean
 cb_idle_reload_configuration (gpointer user_data)
 {
     if (the_manager) {
-        MilterManagerConfiguration *config;
+        GError *error = NULL;
 
-        config = milter_manager_get_configuration(the_manager);
-        if (config)
-            milter_manager_configuration_reload(config);
+        if (!milter_manager_reload(the_manager, &error)) {
+            milter_error("[manager][reload][signal][error] %s",
+                         error->message);
+            g_error_free(error);
+        }
     }
 
     return FALSE;
@@ -274,7 +220,7 @@ reload_configuration_request (int signum)
     if (the_manager) {
         MilterEventLoop *loop;
 
-        loop = milter_agent_get_event_loop(MILTER_AGENT(the_manager));
+        loop = milter_client_get_event_loop(MILTER_CLIENT(the_manager));
         milter_event_loop_add_idle_full(loop,
                                         G_PRIORITY_DEFAULT,
                                         cb_idle_reload_configuration,
@@ -549,27 +495,6 @@ update_max_file_descriptors (MilterManager *manager)
     }
 }
 
-static void
-apply_command_line_options (MilterManagerConfiguration *config)
-{
-    if (option_spec)
-        milter_manager_configuration_set_manager_connection_spec(config,
-                                                                 option_spec);
-    if (option_pid_file)
-        milter_manager_configuration_set_pid_file(config, option_pid_file);
-    if (option_user_name)
-        milter_manager_configuration_set_effective_user(config,
-                                                        option_user_name);
-    if (option_group_name)
-        milter_manager_configuration_set_effective_group(config,
-                                                         option_group_name);
-    if (option_socket_group_name)
-        milter_manager_configuration_set_manager_unix_socket_group(
-            config, option_socket_group_name);
-    if (option_daemon)
-        milter_manager_configuration_set_daemon(config, TRUE);
-}
-
 static void
 append_custom_configuration_directory (MilterManagerConfiguration *config)
 {
@@ -613,9 +538,18 @@ append_custom_configuration_directory (MilterManagerConfiguration *config)
     milter_manager_configuration_append_load_path(config,
                                                   custom_config_directory);
     g_free(custom_config_directory);
+}
 
-    milter_manager_configuration_reload(config);
-    apply_command_line_options(config);
+static void
+load_configuration (MilterManager *manager)
+{
+    GError *error = NULL;
+
+    if (!milter_manager_reload(manager, &error)) {
+        milter_manager_error("[manager][reload][custom-load-path][error] %s",
+                             error->message);
+        g_error_free(error);
+    }
 }
 
 static gchar report_stack_trace_window[4096];
@@ -686,28 +620,34 @@ milter_manager_main (void)
     MilterManagerConfiguration *config;
     MilterEventLoop *loop;
     gboolean remove_socket, daemon;
-    gchar *pid_file = NULL;
     GError *error = NULL;
     struct sigaction report_stack_trace_action;
     struct sigaction shutdown_client_action;
     struct sigaction reload_configuration_request_action;
 
-    config = milter_manager_configuration_new(NULL);
-    if (option_config_dir)
+    manager = the_manager;
+    config = milter_manager_get_configuration(manager);
+
+    if (option_config_dir) {
         milter_manager_configuration_prepend_load_path(config,
                                                        option_config_dir);
-    milter_manager_configuration_reload(config);
-    apply_command_line_options(config);
+        if (!milter_manager_reload(manager, &error)) {
+            milter_manager_error("[manager][configuration][reload]"
+                                 "[command-line-load-path][error] %s",
+                                 error->message);
+            g_error_free(error);
+            error = NULL;
+        }
+    }
     if (getuid() != 0)
         milter_manager_configuration_set_privilege_mode(config, FALSE);
 
-    manager = milter_manager_new(config);
-    g_object_unref(config);
 
     client = MILTER_CLIENT(manager);
     g_signal_connect(client, "error", G_CALLBACK(cb_error), NULL);
 
     append_custom_configuration_directory(config);
+    load_configuration(manager);
 
     if (option_show_config) {
         gchar *dumped_config;
@@ -719,7 +659,6 @@ milter_manager_main (void)
                 g_print("\n");
             g_free(dumped_config);
         }
-        g_object_unref(manager);
         return TRUE;
     }
 
@@ -727,7 +666,6 @@ milter_manager_main (void)
 
     if (milter_manager_configuration_is_privilege_mode(config) &&
         !start_process_launcher_process(manager)) {
-        g_object_unref(manager);
         return FALSE;
     }
 
@@ -737,23 +675,22 @@ milter_manager_main (void)
     if (!milter_client_listen(client, &error)) {
         milter_manager_error("failed to listen: %s", error->message);
         g_error_free(error);
-        g_object_unref(manager);
         return FALSE;
     }
 
     if (!milter_client_drop_privilege(client, &error)) {
         milter_manager_error("failed to drop privilege: %s", error->message);
         g_error_free(error);
-        g_object_unref(manager);
         return FALSE;
     }
 
-    loop = milter_client_get_process_loop(client);
+    loop = milter_client_get_event_loop(client);
     controller = milter_manager_controller_new(manager, loop);
     if (controller && !milter_manager_controller_listen(controller, &error)) {
         milter_manager_error("failed to listen controller socket: %s",
                              error->message);
         g_error_free(error);
+        error = NULL;
         g_object_unref(controller);
         controller = NULL;
     }
@@ -767,26 +704,10 @@ milter_manager_main (void)
             g_error_free(error);
             if (controller)
                 g_object_unref(controller);
-            g_object_unref(manager);
             return FALSE;
         }
     }
 
-    pid_file = g_strdup(milter_manager_configuration_get_pid_file(config));
-    if (pid_file) {
-        gchar *content;
-        GError *error = NULL;
-
-        content = g_strdup_printf("%u\n", getpid());
-        if (!g_file_set_contents(pid_file, content, -1, &error)) {
-            milter_error("[manager][error][pid][save] %s: %s",
-                         pid_file, error->message);
-            g_error_free(error);
-            g_free(pid_file);
-            pid_file = NULL;
-        }
-    }
-
     the_manager = manager;
 
 #define SETUP_SIGNAL_ACTION(handler)            \
@@ -832,18 +753,6 @@ milter_manager_main (void)
     if (controller)
         g_object_unref(controller);
 
-    if (the_manager) {
-        g_object_unref(the_manager);
-        the_manager = NULL;
-    }
-
-    if (pid_file) {
-        if (g_unlink(pid_file) == -1)
-            milter_error("[manager][error][pid][remove] %s: %s",
-                         pid_file, g_strerror(errno));
-        g_free(pid_file);
-    }
-
     return TRUE;
 }
 
