@@ -40,7 +40,6 @@ typedef struct _MilterManagerPrivate MilterManagerPrivate;
 struct _MilterManagerPrivate
 {
     MilterManagerConfiguration *configuration;
-    MilterSyslogLogger *logger;
     GList *leaders;
     GList *next_connection_checked_leader;
     gboolean connection_checking;
@@ -52,6 +51,9 @@ struct _MilterManagerPrivate
     guint current_periodical_connection_check_interval;
 
     GList *finished_leaders;
+
+    gboolean is_custom_n_workers;
+    gboolean is_custom_run_as_daemon;
 };
 
 enum
@@ -74,12 +76,15 @@ static void get_property   (GObject         *object,
 
 static void   connection_established      (MilterClient *client,
                                            MilterClientContext *context);
-static gchar *get_default_connection_spec (MilterClient *client);
+static const gchar *get_connection_spec   (MilterClient *client);
+static gboolean     set_connection_spec   (MilterClient *client,
+                                           const gchar  *spec,
+                                           GError      **error);
 static guint  get_unix_socket_mode        (MilterClient *client);
 static void   set_unix_socket_mode        (MilterClient *client,
                                            guint         mode);
 static const gchar *get_unix_socket_group (MilterClient *client);
-static void   set_unix_socket_group       (MilterClient *client,
+static void         set_unix_socket_group (MilterClient *client,
                                            const gchar  *group);
 static gboolean is_remove_unix_socket_on_close (MilterClient *client);
 static guint  get_suspend_time_on_unacceptable
@@ -91,10 +96,10 @@ static guint  get_max_connections         (MilterClient *client);
 static void   set_max_connections         (MilterClient *client,
                                            guint         max_connections);
 static const gchar *get_effective_user    (MilterClient *client);
-static void   set_effective_user          (MilterClient *client,
+static void         set_effective_user    (MilterClient *client,
                                            const gchar  *effective_user);
 static const gchar *get_effective_group   (MilterClient *client);
-static void   set_effective_group         (MilterClient *client,
+static void         set_effective_group   (MilterClient *client,
                                            const gchar  *effective_group);
 static guint  get_maintenance_interval    (MilterClient *client);
 static void   set_maintenance_interval    (MilterClient *client,
@@ -116,6 +121,12 @@ static guint  get_default_packet_buffer_size
 static void   set_default_packet_buffer_size
                                           (MilterClient *client,
                                            guint         size);
+static const gchar *get_pid_file          (MilterClient *client);
+static void   set_pid_file                (MilterClient *client,
+                                           const gchar  *pid_file);
+static gboolean is_run_as_daemon          (MilterClient *client);
+static void   set_run_as_daemon           (MilterClient *client,
+                                           gboolean      daemon);
 
 static void
 milter_manager_class_init (MilterManagerClass *klass)
@@ -132,7 +143,8 @@ milter_manager_class_init (MilterManagerClass *klass)
     gobject_class->get_property = get_property;
 
     client_class->connection_established      = connection_established;
-    client_class->get_default_connection_spec = get_default_connection_spec;
+    client_class->get_connection_spec         = get_connection_spec;
+    client_class->set_connection_spec         = set_connection_spec;
     client_class->get_unix_socket_mode        = get_unix_socket_mode;
     client_class->set_unix_socket_mode        = set_unix_socket_mode;
     client_class->get_unix_socket_group       = get_unix_socket_group;
@@ -156,6 +168,10 @@ milter_manager_class_init (MilterManagerClass *klass)
         get_default_packet_buffer_size;
     client_class->set_default_packet_buffer_size =
         set_default_packet_buffer_size;
+    client_class->get_pid_file = get_pid_file;
+    client_class->set_pid_file = set_pid_file;
+    client_class->is_run_as_daemon = is_run_as_daemon;
+    client_class->set_run_as_daemon = set_run_as_daemon;
     client_class->maintain = maintain;
     client_class->sessions_finished = sessions_finished;
 
@@ -181,7 +197,6 @@ milter_manager_init (MilterManager *manager)
     priv->leaders = NULL;
     priv->next_connection_checked_leader = NULL;
     priv->connection_checking = FALSE;
-    priv->logger = milter_syslog_logger_new("milter-manager", NULL);
 
     priv->launcher_read_channel = NULL;
     priv->launcher_write_channel = NULL;
@@ -211,7 +226,7 @@ dispose_periodical_connection_checker (MilterManager *manager)
     if (priv->periodical_connection_checker_id == 0)
         return;
 
-    loop = milter_client_get_process_loop(MILTER_CLIENT(manager));
+    loop = milter_client_get_event_loop(MILTER_CLIENT(manager));
     milter_event_loop_remove(loop, priv->periodical_connection_checker_id);
     priv->periodical_connection_checker_id = 0;
 }
@@ -260,11 +275,6 @@ dispose (GObject *object)
     }
     priv->next_connection_checked_leader = NULL;
 
-    if (priv->logger) {
-        g_object_unref(priv->logger);
-        priv->logger = NULL;
-    }
-
     milter_manager_set_launcher_channel(MILTER_MANAGER(object), NULL, NULL);
 
     G_OBJECT_CLASS(milter_manager_parent_class)->dispose(object);
@@ -323,6 +333,9 @@ milter_manager_new (MilterManagerConfiguration *configuration)
 {
     return g_object_new(MILTER_TYPE_MANAGER,
                         "configuration", configuration,
+                        "syslog-identify", "milter-manager",
+                        "syslog-facility", "mail",
+                        "start-syslog", TRUE,
                         NULL);
 }
 
@@ -380,7 +393,7 @@ start_periodical_connection_checker (MilterManager *manager)
                          "initial" : "update");
             dispose_periodical_connection_checker(manager);
             priv->current_periodical_connection_check_interval = interval;
-            loop = milter_client_get_process_loop(MILTER_CLIENT(manager));
+            loop = milter_client_get_event_loop(MILTER_CLIENT(manager));
             priv->periodical_connection_checker_id =
                 milter_event_loop_add_timeout(loop, interval,
                                               connection_check, manager);
@@ -791,8 +804,8 @@ connection_established (MilterClient *client, MilterClientContext *context)
     start_periodical_connection_checker(manager);
 }
 
-static gchar *
-get_default_connection_spec (MilterClient *client)
+static const gchar *
+get_connection_spec (MilterClient *client)
 {
     MilterManager *manager;
     MilterManagerPrivate *priv;
@@ -805,40 +818,75 @@ get_default_connection_spec (MilterClient *client)
     spec =
         milter_manager_configuration_get_manager_connection_spec(configuration);
     if (spec) {
-        return g_strdup(spec);
+        return spec;
     } else {
         MilterClientClass *klass;
 
         klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
-        return klass->get_default_connection_spec(MILTER_CLIENT(manager));
+        return klass->get_connection_spec(MILTER_CLIENT(manager));
     }
 }
 
+static gboolean
+set_connection_spec (MilterClient *client, const gchar *spec, GError **error)
+{
+    MilterClientClass *klass;
+    MilterManager *manager;
+    gboolean success;
+
+    manager = MILTER_MANAGER(client);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    success = klass->set_connection_spec(MILTER_CLIENT(manager), spec, error);
+    if (success) {
+        MilterManagerPrivate *priv;
+        MilterManagerConfiguration *configuration;
+
+        priv = MILTER_MANAGER_GET_PRIVATE(manager);
+        configuration = priv->configuration;
+        milter_manager_configuration_set_manager_connection_spec(configuration,
+                                                                 spec);
+    }
+    return success;
+}
+
 static guint
 get_unix_socket_mode (MilterClient *client)
 {
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
+    guint mode;
 
     manager = MILTER_MANAGER(client);
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
-    return milter_manager_configuration_get_manager_unix_socket_mode(configuration);
+    mode =
+        milter_manager_configuration_get_manager_unix_socket_mode(configuration);
+    if (mode == 0) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        mode = klass->get_unix_socket_mode(client);
+    }
+    return mode;
 }
 
 static void
 set_unix_socket_mode (MilterClient *client, guint mode)
 {
+    MilterClientClass *klass;
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
 
     manager = MILTER_MANAGER(client);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_unix_socket_mode(client, mode);
+
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
-    return milter_manager_configuration_set_manager_unix_socket_mode(configuration,
-                                                                     mode);
+    milter_manager_configuration_set_manager_unix_socket_mode(configuration,
+                                                              mode);
 }
 
 static const gchar *
@@ -847,21 +895,33 @@ get_unix_socket_group (MilterClient *client)
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
+    const gchar *group;
 
     manager = MILTER_MANAGER(client);
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
-    return milter_manager_configuration_get_manager_unix_socket_group(configuration);
+    group = milter_manager_configuration_get_manager_unix_socket_group(configuration);
+    if (!group) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        group = klass->get_unix_socket_group(client);
+    }
+    return group;
 }
 
 static void
 set_unix_socket_group (MilterClient *client, const gchar *group)
 {
+    MilterClientClass *klass;
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
 
     manager = MILTER_MANAGER(client);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_unix_socket_group(client, group);
+
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
     return milter_manager_configuration_set_manager_unix_socket_group(configuration,
@@ -941,21 +1001,34 @@ get_effective_user (MilterClient *client)
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
+    const gchar *effective_user;
 
     manager = MILTER_MANAGER(client);
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
-    return milter_manager_configuration_get_effective_user(configuration);
+    effective_user =
+        milter_manager_configuration_get_effective_user(configuration);
+    if (!effective_user) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        effective_user = klass->get_effective_user(client);
+    }
+    return effective_user;
 }
 
 static void
 set_effective_user (MilterClient *client, const gchar *effective_user)
 {
+    MilterClientClass *klass;
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
 
     manager = MILTER_MANAGER(client);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_effective_user(client, effective_user);
+
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
     milter_manager_configuration_set_effective_user(configuration,
@@ -968,22 +1041,35 @@ get_effective_group (MilterClient *client)
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
+    const gchar *effective_group;
 
     manager = MILTER_MANAGER(client);
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
     configuration = priv->configuration;
-    return milter_manager_configuration_get_effective_group(configuration);
+    effective_group =
+        milter_manager_configuration_get_effective_group(configuration);
+    if (!effective_group) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        effective_group = klass->get_effective_group(client);
+    }
+    return effective_group;
 }
 
 static void
 set_effective_group (MilterClient *client, const gchar *effective_group)
 {
+    MilterClientClass *klass;
     MilterManager *manager;
     MilterManagerPrivate *priv;
     MilterManagerConfiguration *configuration;
 
     manager = MILTER_MANAGER(client);
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_effective_group(client, effective_group);
+
     configuration = priv->configuration;
     milter_manager_configuration_set_effective_group(configuration,
                                                      effective_group);
@@ -1043,6 +1129,44 @@ set_default_packet_buffer_size (MilterClient *client, guint size)
                                                                 size);
 }
 
+static const gchar *
+get_pid_file (MilterClient *client)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+    const gchar *pid_file;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    pid_file = milter_manager_configuration_get_pid_file(configuration);
+    if (!pid_file) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        pid_file = klass->get_pid_file(client);
+    }
+    return pid_file;
+}
+
+static void
+set_pid_file (MilterClient *client, const gchar *pid_file)
+{
+    MilterClientClass *klass;
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_pid_file(client, pid_file);
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    milter_manager_configuration_set_pid_file(configuration, pid_file);
+}
+
 static void
 maintain (MilterClient *client)
 {
@@ -1068,41 +1192,123 @@ sessions_finished (MilterClient *client, guint n_finished_sessions)
 static guint
 get_n_workers (MilterClient *client)
 {
+    MilterManager *manager;
     MilterManagerPrivate *priv;
+    guint n_workers;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(client);
-    return milter_manager_configuration_get_n_workers(priv->configuration);
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    if (priv->is_custom_n_workers) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        n_workers = klass->get_n_workers(client);
+    } else {
+        MilterManagerConfiguration *configuration;
+
+        configuration = priv->configuration;
+        n_workers = milter_manager_configuration_get_n_workers(configuration);
+    }
+    return n_workers;
 }
 
 static void
 set_n_workers (MilterClient *client, guint n_workers)
 {
+    MilterClientClass *klass;
+    MilterManager *manager;
     MilterManagerPrivate *priv;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    manager = MILTER_MANAGER(client);
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    priv->is_custom_n_workers = TRUE;
+
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_n_workers(client, n_workers);
     milter_manager_configuration_set_n_workers(priv->configuration, n_workers);
 }
 
 static MilterClientEventLoopBackend
 get_event_loop_backend (MilterClient *client)
 {
-    MilterManagerPrivate *priv;
+    MilterClientClass *klass;
+    MilterManager *manager;
+    MilterClientEventLoopBackend backend;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(client);
-    return milter_manager_configuration_get_event_loop_backend(priv->configuration);
+    manager = MILTER_MANAGER(client);
+
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    backend = klass->get_event_loop_backend(client);
+    if (backend == MILTER_CLIENT_EVENT_LOOP_BACKEND_DEFAULT) {
+        MilterManagerPrivate *priv;
+        MilterManagerConfiguration *config;
+
+        priv = MILTER_MANAGER_GET_PRIVATE(manager);
+        config = priv->configuration;
+        backend = milter_manager_configuration_get_event_loop_backend(config);
+    }
+    return backend;
 }
 
 static void
 set_event_loop_backend (MilterClient *client,
                         MilterClientEventLoopBackend backend)
 {
+    MilterClientClass *klass;
+    MilterManager *manager;
     MilterManagerPrivate *priv;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    manager = MILTER_MANAGER(client);
+
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_event_loop_backend(client, backend);
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
     milter_manager_configuration_set_event_loop_backend(priv->configuration,
                                                         backend);
 }
 
+static gboolean
+is_run_as_daemon (MilterClient *client)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    gboolean run_as_daemon;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    if (priv->is_custom_run_as_daemon) {
+        MilterClientClass *klass;
+
+        klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+        run_as_daemon = klass->is_run_as_daemon(client);
+    } else {
+        MilterManagerConfiguration *configuration;
+
+        configuration = priv->configuration;
+        run_as_daemon = milter_manager_configuration_is_daemon(configuration);
+    }
+    return run_as_daemon;
+}
+
+static void
+set_run_as_daemon (MilterClient *client, gboolean run_as_daemon)
+{
+    MilterClientClass *klass;
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+
+    manager = MILTER_MANAGER(client);
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    priv->is_custom_run_as_daemon = TRUE;
+
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+    klass->set_run_as_daemon(client, run_as_daemon);
+    milter_manager_configuration_set_daemon(priv->configuration, run_as_daemon);
+}
+
 MilterManagerConfiguration *
 milter_manager_get_configuration (MilterManager *manager)
 {
@@ -1115,6 +1321,130 @@ milter_manager_get_leaders (MilterManager *manager)
     return MILTER_MANAGER_GET_PRIVATE(manager)->leaders;
 }
 
+static void
+apply_syslog_parameters (MilterManager *manager)
+{
+    MilterClient *client;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    client = MILTER_CLIENT(manager);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+
+    if (milter_manager_configuration_get_use_syslog(configuration)) {
+        if (milter_client_get_syslog_enabled(client)) {
+            const gchar *facility;
+            facility =
+                milter_manager_configuration_get_syslog_facility(configuration);
+            milter_client_set_syslog_facility(client, facility);
+        } else {
+            milter_client_start_syslog(client);
+        }
+    } else {
+        milter_client_stop_syslog(client);
+    }
+}
+
+static void
+apply_custom_parameters (MilterManager *manager)
+{
+    MilterClient *client;
+    MilterClientClass *klass;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+    const gchar *spec;
+    const gchar *pid_file;
+    const gchar *effective_user;
+    const gchar *effective_group;
+    guint unix_socket_mode;
+    const gchar *unix_socket_group;
+    MilterClientEventLoopBackend backend;
+
+    client = MILTER_CLIENT(manager);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+
+    klass = MILTER_CLIENT_CLASS(milter_manager_parent_class);
+
+    spec = klass->get_connection_spec(client);
+    if (spec) {
+        milter_manager_configuration_set_manager_connection_spec(configuration,
+                                                                 spec);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.connection_spec");
+    }
+
+    pid_file = klass->get_pid_file(client);
+    if (pid_file) {
+        milter_manager_configuration_set_pid_file(configuration, pid_file);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.pid_file");
+    }
+
+    effective_user = klass->get_effective_user(client);
+    if (effective_user) {
+        milter_manager_configuration_set_effective_user(configuration,
+                                                        effective_user);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "security.effective_user");
+    }
+
+    effective_group = klass->get_effective_group(client);
+    if (effective_group) {
+        milter_manager_configuration_set_effective_group(configuration,
+                                                         effective_group);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "security.effective_grourp");
+    }
+
+    unix_socket_mode = klass->get_unix_socket_mode(client);
+    if (unix_socket_mode) {
+        milter_manager_configuration_set_manager_unix_socket_mode(configuration,
+                                                                  unix_socket_mode);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.unix_socket_mode");
+    }
+
+    unix_socket_group = klass->get_unix_socket_group(client);
+    if (unix_socket_group) {
+        milter_manager_configuration_set_manager_unix_socket_group(configuration,
+                                                                   unix_socket_group);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.unix_socket_group");
+    }
+
+    if (priv->is_custom_n_workers) {
+        guint n_workers;
+
+        n_workers = klass->get_n_workers(client);
+        milter_manager_configuration_set_n_workers(configuration, n_workers);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.n_workers");
+    }
+
+    backend = klass->get_event_loop_backend(client);
+    if (backend != MILTER_CLIENT_EVENT_LOOP_BACKEND_DEFAULT) {
+        milter_manager_configuration_set_event_loop_backend(configuration,
+                                                            backend);
+        milter_manager_configuration_reset_location(configuration,
+                                                    "manager.event_loop_backend");
+    }
+}
+
+gboolean
+milter_manager_reload (MilterManager *manager, GError **error)
+{
+    MilterManagerPrivate *priv;
+    gboolean success;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    success = milter_manager_configuration_reload(priv->configuration, error);
+    apply_syslog_parameters(manager);
+    apply_custom_parameters(manager);
+    return success;
+}
+
 void
 milter_manager_set_launcher_channel (MilterManager *manager,
                                      GIOChannel *read_channel,
