@@ -114,7 +114,7 @@ teardown (void)
 #define milter_assert_equal_log_level(flags, name)                      \
     gcut_assert_equal_flags(MILTER_TYPE_LOG_LEVEL_FLAGS,                \
                             flags,                                      \
-                            milter_log_level_flags_from_string(name))
+                            milter_log_level_flags_from_string(name, NULL))
 
 void
 test_level_from_string (void)
@@ -256,7 +256,11 @@ test_interesting_level (void)
 {
     logger = milter_logger_new();
     gcut_assert_equal_flags(MILTER_TYPE_LOG_LEVEL_FLAGS,
-                            MILTER_LOG_LEVEL_ERROR | MILTER_LOG_LEVEL_CRITICAL,
+                            MILTER_LOG_LEVEL_MESSAGE |
+                            MILTER_LOG_LEVEL_WARNING |
+                            MILTER_LOG_LEVEL_ERROR |
+                            MILTER_LOG_LEVEL_CRITICAL |
+                            MILTER_LOG_LEVEL_STATISTICS,
                             milter_logger_get_interesting_level(logger));
 
     milter_logger_set_target_level(logger,
