@@ -17,7 +17,6 @@
  *
  */
 
-
 #include <milter/manager/milter-manager-configuration.h>
 #include <milter/manager/milter-manager-children.h>
 
@@ -56,6 +55,9 @@ void test_connection_check_interval (void);
 void test_location (void);
 void test_n_workers (void);
 void test_default_packet_buffer_size (void);
+void test_prefix (void);
+void test_use_syslog (void);
+void test_syslog_facility (void);
 void test_egg (void);
 void test_find_egg (void);
 void test_remove_egg (void);
@@ -208,8 +210,11 @@ test_children (void)
 
     child = milter_manager_egg_hatch(egg);
     milter_manager_children_add_child(expected_children, child);
+    g_object_unref(child);
 
     cut_assert_false(attached_to);
+    if (actual_children)
+        g_object_unref(actual_children);
     actual_children = milter_manager_children_new(config, loop);
     milter_manager_configuration_setup_children(config, actual_children, NULL);
     milter_assert_equal_children(expected_children, actual_children);
@@ -571,6 +576,37 @@ test_default_packet_buffer_size (void)
         milter_manager_configuration_get_default_packet_buffer_size(config));
 }
 
+void
+test_prefix (void)
+{
+    cut_assert_equal_string(PREFIX,
+                            milter_manager_configuration_get_prefix(config));
+}
+
+void
+test_use_syslog (void)
+{
+    cut_assert_equal_boolean(
+        TRUE,
+        milter_manager_configuration_get_use_syslog(config));
+    milter_manager_configuration_set_use_syslog(config, FALSE);
+    cut_assert_equal_boolean(
+        FALSE,
+        milter_manager_configuration_get_use_syslog(config));
+}
+
+void
+test_syslog_facility (void)
+{
+    cut_assert_equal_string(
+        NULL,
+        milter_manager_configuration_get_syslog_facility(config));
+    milter_manager_configuration_set_syslog_facility(config, "local1");
+    cut_assert_equal_string(
+        "local1",
+        milter_manager_configuration_get_syslog_facility(config));
+}
+
 static void
 milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
 {
@@ -655,6 +691,13 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
         0,
         milter_manager_configuration_get_default_packet_buffer_size(config));
 
+    cut_assert_equal_boolean(
+        TRUE,
+        milter_manager_configuration_get_use_syslog(config));
+    cut_assert_equal_string(
+        NULL,
+        milter_manager_configuration_get_syslog_facility(config));
+
     if (expected_children)
         g_object_unref(expected_children);
     expected_children = milter_manager_children_new(config, loop);
@@ -853,6 +896,7 @@ void
 test_clear (void)
 {
     gulong handler_id;
+    GError *error = NULL;
 
     milter_assert_default_configuration(config);
 
@@ -870,15 +914,19 @@ test_clear (void)
     test_suspend_time_on_unacceptable();
     test_max_connections();
     test_max_file_descriptors();
+    test_event_loop_backend();
     test_connection_check_interval();
     test_n_workers();
     test_default_packet_buffer_size();
+    test_use_syslog();
+    test_syslog_facility();
 
     handler_id = g_signal_connect(config, "connected",
                                   G_CALLBACK(cb_connected), NULL);
     cut_assert_true(g_signal_handler_is_connected(config, handler_id));
 
-    milter_manager_configuration_clear(config);
+    milter_manager_configuration_clear(config, &error);
+    gcut_assert_error(error);
     milter_assert_default_configuration(config);
 
     cut_assert_false(g_signal_handler_is_connected(config, handler_id));
